from io import StringIO
from loguru import logger
import psycopg2
import psycopg2.extensions
from AbstractSinkHandler import AbstractSinkHandler
from PgDbHandle import PgDbHandle, PgDbHandleException


class TsDbSinkException(Exception):
    def __init__(self, pgDbHandleException):
        super().__init__()
        self.errType = pgDbHandleException.errType
        self.code = pgDbHandleException.code
        self.msg = pgDbHandleException.msg

def _copy(cursor, table, columns, data):
    stringIO = StringIO(data)
    cursor.copy_from(stringIO, table, columns=columns)

def _insert(cursor, table, columns, data):
    stmt = f"insert into {table} ({','.join(columns)}) values ({','.join(['%s'] * len(columns))})"
    cursor.execute(stmt, data)

class AbstractTsDbSinkHandler(AbstractSinkHandler):
    def __init__(self, config, name, inQueue, experiment):
        super().__init__(config, name, inQueue, experiment)
        logger.info("constructor called")
        self.dbh = PgDbHandle(config)

    def init(self):
        logger.info("init")
        self.dbh.connect()

    def deinit(self):
        logger.info("deinit")
        self.dbh.disconnect()
    
    def copy(self, table, columns, data):
        try: 
            self.dbh.execute(_copy, table, columns, data)
        except PgDbHandleException as e:
            raise TsDbSinkException(e)
    
    def insert(self, table, columns, data):
        try: 
            self.dbh.execute(_insert, table, columns, data)
        except PgDbHandleException as e:
            raise TsDbSinkException(e)
